@@ -58,6 +58,12 @@ class Compiler:
             # watbia-ctl {args}
             "^watiba-ctl \s*(\S.*)": self.watiba_ctl_args,
 
+            # hook-cmd "command pattern" {function: {parms}, function: {parms}}
+            ".*?hook-cmd \s*(\S.*) (\S.*) (\S.*)": self.hook_generator,
+
+           # remove-hooks
+            ".*?remove-hooks\s.*?(\S.*)?$": self.remove_hooks_generator,
+
             # chain {host:cmd...
             "^(\S.*)?chain \s*`(\S.*)` \s*(\S.*)": self.chain_generator,
 
@@ -93,6 +99,18 @@ class Compiler:
                 self.last_stmt = self.current_statement if self.current_statement.lstrip()[
                                                                0] not in nothingness else self.last_stmt
 
+
+    # Generate hook-cmd command
+    def hook_generator(self, parms):
+        self.output.append(f'{parms["indentation"]}{watiba_ref}.spawn_ctlr.add_hook({parms["match"].group(1)}, {parms["match"].group(2)}, {parms["match"].group(3)})')
+    
+    # Remove command hooks
+    def remove_hooks_generator(self, parms):
+        if len(parms["match"].groups()) >  0 and parms["match"].group(1) != None:
+            self.output.append(f'{parms["indentation"]}{watiba_ref}.spawn_ctlr.remove_hooks(parms["match"].group(1))')
+        else:
+            self.output.append(f'{parms["indentation"]}{watiba_ref}.spawn_ctlr.remove_hooks()')
+
     # Generate chain command
     def chain_generator(self, parms):
         assignment = parms["match"].group(1) if parms["match"].group(1) else ""
@@ -245,7 +263,7 @@ if __name__ == "__main__":
         sys.exit(0)
 
     # the versions array is generated at build time (see this module in bin/)
-    if sys.argv[1] == "version":
+    if sys.argv[1] == "version" or sys.argv[1] == "--version":
         for v in versions:
             print(v)
         sys.exit(0)
