@@ -28,7 +28,7 @@ class WTChainException(Exception):
 ########################################## Watiba #############################################################
 ###############################################################################################################
 # Singleton object with no side effects
-# Executes the command an returns a new WTOutput object
+# Executes the command and returns a new WTOutput object
 class Watiba(Exception):
 
     def __init__(self):
@@ -101,17 +101,68 @@ class Watiba(Exception):
                 and inspect.ismethod(getattr(parent_locals['promise'], "resolved")):
             # Link this child promise to its parent
             l_promise.relate(parent_locals['promise'])
+        
+        # Run all the command pre-execution hooks
+        # If any hooks returns False, meaning it somehow failed (that's determined by the hook)
+        # then report so an exception is thrown by the caller
+        def run_hooks(hooks, command):
 
+            # object returned to caller:
+            #  "success" - Aggregate True/False of all hooks.  (i.e. any hook that reports False will cause this value to return False)
+            #  "failed-hooks" - Array of hook names that reported a False condition
+            #       Note: all hooks are called in order even after one reports failure (i.e. reports False)
+            return_obj = {"success": True, "failed-hooks": []}
+
+            # Loop through the hooks and run them.  Also track ones that fail (i.e. report a False return code)
+            for cmd_regex, functions in hooks.items():
+
+                # Check if the command passed to us matches the regex expression
+                mat = re.match(cmd_regex, command)
+
+                # Does this command have attached hooks?
+                if mat:
+
+                    # Yes, command has hooks.  Run them.
+                    # If the hook fails track it, but keep going with the other hooks
+                    for func, parms in functions.items():
+
+                        # Call the hook.  The hook must return True if succeeded, False if failed
+                        rc = func(mat, parms)
+
+                        # If caller's hook didn't return a bool value, then it is marked as failed
+                        rc = False if type(rc) != bool else rc
+
+                        # Track failed hooks
+                        if rc == False:
+                            return_obj["failed-hooks"].append(func.__name__)
+                    
+                        return_obj["success"] &= rc
+            
+            return return_obj
+
+
+        # This is run under the new thread, and under the control of wtspawncontroller.py (i.e. spawn controller calls this function)
         def run_command(promise, thread_args):
             # Get our thread id
             promise.thread_id = threading.get_ident()
 
+            # Run hooks, if any were defined
+            if len(thread_args["spawn-args"]["hooks"]) > 0:
+                ro = run_hooks(thread_args["spawn-args"]["hooks"], thread_args["command"])
+
+                # If any hook failed, throw an exception to our caller and name the bad hooks
+                if ro["success"] != True:
+                    raise Exception(f"These hooks failed: {', '.join(ro['failed-hooks'])}")
+
+
             # Execute the command in a new thread (this is synchronously run)
             promise.output = self.execute(thread_args["command"], thread_args["host"])
 
             # Call promise resolver
             promise.set_resolution(thread_args["resolver"](promise, copy.copy(thread_args["spawn-args"])))
 
+
+        # Call wtspawncontroller.py to run the command under a new thread
         try:
             thread_args = {"command": command, "resolver": resolver, "spawn-args": spawn_args, "host": host}
 
@@ -122,6 +173,8 @@ class Watiba(Exception):
             print(f"ERROR.  w_async thread execution failed. {ex.promise.command}")
 
         return l_promise
+    
+
 
     # Pipe either stdout or stderr to some target host with some target cmd
     def pipe(self, pipe_source, pipe_target):
